- Updated the address for the FSF in the opening comment.
[rsync/rsync.git] / main.c
diff --git a/main.c b/main.c
index c82453e..1237447 100644 (file)
--- a/main.c
+++ b/main.c
@@ -1,23 +1,25 @@
-/* -*- c-file-style: "linux" -*-
-
-   Copyright (C) 1996-2001 by Andrew Tridgell <tridge@samba.org>
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+/*
+ * The startup routines, including main(), for rsync.
+ *
+ * Copyright (C) 1996-2001 Andrew Tridgell <tridge@samba.org>
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include "rsync.h"
 #if defined CONFIG_LOCALE && defined HAVE_LOCALE_H
@@ -40,8 +42,8 @@ extern int kluge_around_eof;
 extern int do_stats;
 extern int log_got_error;
 extern int module_id;
-extern int orig_umask;
 extern int copy_links;
+extern int copy_dirlinks;
 extern int keep_dirlinks;
 extern int preserve_hard_links;
 extern int protocol_version;
@@ -62,6 +64,7 @@ extern char *shell_cmd;
 extern char *batch_name;
 
 int local_server = 0;
+mode_t orig_umask = 0;
 struct file_list *the_file_list;
 
 /* There's probably never more than at most 2 outstanding child processes,
@@ -188,7 +191,7 @@ static void handle_stats(int f)
        /* this is the client */
 
        if (f < 0 && !am_sender) /* e.g. when we got an empty file list. */
-               ; 
+               ;
        else if (!am_sender) {
                /* Read the first two in opposite order because the meaning of
                 * read/write swaps when switching from sender to receiver. */
@@ -437,8 +440,8 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char *path,
 
 /* The receiving side operates in one of two modes:
  *
* 1. it enters a directory and receives one or more files, placing them
* according to their names in the file-list.
  *
  * 2. it receives a single file and saves it using the name in the
  * destination path instead of its file-list name.  This requires a
@@ -457,7 +460,7 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
                        flist->count, NS(dest_path));
        }
 
-       if (!dest_path)
+       if (!dest_path || list_only)
                return NULL;
 
        /* If the destination path refers to an existing directory, enter
@@ -479,6 +482,17 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
                                " copying more than 1 file\n");
                        exit_cleanup(RERR_FILESELECT);
                }
+               /* Caution: flist->count could be 0! */
+               if (flist->count == 1 && S_ISDIR(flist->files[0]->mode)) {
+                       rprintf(FERROR,
+                               "ERROR: cannot overwrite non-directory"
+                               " with a directory\n");
+                       exit_cleanup(RERR_FILESELECT);
+               }
+       } else if (errno != ENOENT) {
+               rsyserr(FERROR, errno, "cannot stat destination %s",
+                       full_fname(dest_path));
+               exit_cleanup(RERR_FILESELECT);
        }
 
        cp = strrchr(dest_path, '/');
@@ -491,13 +505,11 @@ static char *get_local_name(struct file_list *flist, char *dest_path)
                if (cp && !cp[1])
                        *cp = '\0';
 
-               umask(orig_umask);
-               if (do_mkdir(dest_path, 0777) != 0) {
+               if (mkdir_defmode(dest_path) != 0) {
                        rsyserr(FERROR, errno, "mkdir %s failed",
                                full_fname(dest_path));
                        exit_cleanup(RERR_FILEIO);
                }
-               umask(0);
 
                if (verbose)
                        rprintf(FINFO, "created directory %s\n", dest_path);
@@ -627,7 +639,7 @@ static int do_recv(int f_in,int f_out,struct file_list *flist,char *local_name)
 
        /* The receiving side mustn't obey this, or an existing symlink that
         * points to an identical file won't be replaced by the referent. */
-       copy_links = 0;
+       copy_links = copy_dirlinks = 0;
 
        if (preserve_hard_links)
                init_hard_links();
@@ -960,12 +972,12 @@ static int start_client(int argc, char *argv[])
                return rc;
 
        if (!read_batch) { /* for read_batch, NO source is specified */
-               argc--;
                shell_path = check_for_hostspec(argv[0], &shell_machine, &rsync_port);
                if (shell_path) { /* source is remote */
                        char *dummy1;
                        int dummy2;
-                       if (argc && check_for_hostspec(argv[argc], &dummy1, &dummy2)) {
+                       if (--argc
+                        && check_for_hostspec(argv[argc], &dummy1, &dummy2)) {
                                rprintf(FERROR,
                                        "The source and destination cannot both be remote.\n");
                                exit_cleanup(RERR_SYNTAX);
@@ -990,12 +1002,14 @@ static int start_client(int argc, char *argv[])
                } else { /* source is local, check dest arg */
                        am_sender = 1;
 
-                       if (argc < 1) { /* destination required */
-                               usage(FERROR);
-                               exit_cleanup(RERR_SYNTAX);
+                       if (argc > 1)
+                               p = argv[--argc];
+                       else {
+                               p = ".";
+                               list_only = 1;
                        }
 
-                       shell_path = check_for_hostspec(argv[argc], &shell_machine, &rsync_port);
+                       shell_path = check_for_hostspec(p, &shell_machine, &rsync_port);
                        if (shell_path && filesfrom_host && *filesfrom_host
                            && strcmp(filesfrom_host, shell_machine) != 0) {
                                rprintf(FERROR,
@@ -1010,7 +1024,7 @@ static int start_client(int argc, char *argv[])
                                        exit_cleanup(RERR_SYNTAX);
                                }
                                shell_machine = NULL;
-                               shell_path = argv[argc];
+                               shell_path = p;
                        } else if (rsync_port) {
                                if (!shell_cmd) {
                                        return start_socket_client(shell_machine,
@@ -1206,7 +1220,7 @@ int main(int argc,char *argv[])
 
        /* we set a 0 umask so that correct file permissions can be
         * carried across */
-       orig_umask = (int)umask(0);
+       orig_umask = umask(0);
 
 #if defined CONFIG_LOCALE && defined HAVE_SETLOCALE
        setlocale(LC_CTYPE, "");
@@ -437,8 +440,8 @@ static pid_t do_cmd(char *cmd, char *machine, char *user, char *path,
 /* The receiving side operates in one of two modes:
  *
- * 1. it enters a directory and receives one or more files, placing them
- * according to their names in the file-list.
+ * 1. it receives any number of files into a destination directory,
+ * placing them according to their names in the file-list.
  *
  * 2. it receives a single file and saves it using the name in the